#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include "trees.h"
#include "rings_start.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif /* WITH_DMALLOC */


void get_rings_starts(IMAGE_BYTE *iMag, MOSAIC_INDEX *index, int thres, int mrows, int ncols, int joffset, int **finalstarts)
{
  int *count=NULL;
  int *tempcount=NULL;
  int i,j,k,jstart;
  int gradient, kplus, kminus, kmax;
 
  IMAGE_BYTE *startpoints=NULL;

  count=(int *)calloc(index->npix,sizeof(int)); /*count value in m*n window, initialized to 0*/
  tempcount=(int *)calloc(index->npix,sizeof(int));
  GETMEM_BYTE(iMag->nbnd, mrows, iMag->npix, &startpoints);/*possible starts, to be counted in the m*n window*/
  
  /*estimate possible starts as one that cross the threshold*/
  /*simple implementation*/
  for(k=index->hstart+1;k<=index->hend;k++) {
    jstart=index->vstart[k] - index->voffset[k]+joffset;
    for(j=0;j<mrows;j++) {
      if(iMag->data[0][j+jstart][k]>thres) {
	startpoints->data[0][j][k]=1;
	tempcount[k]++;
      }
    }
  }
  
  for(k=index->hstart+1;k<index->hend;k++)
    for(j=k-ncols/2;j<=k+ncols/2;j++) {
      count[k]+=tempcount[j];
    }
  
  /*find the maximas in the count values*/
  kplus=-1;
  kminus=-1;
  for(k=index->hstart+1;k<index->hend;k++) {
    gradient=count[k+1]-count[k];
    if(gradient>0) 
      kplus=k;
    else if(gradient < 0) {
      kminus=k;
      if(kplus>0 && kplus < kminus) { /*register a maxima*/
	kmax=(kminus+kplus+1)/2;
	kplus=-1;
	*(*finalstarts+kmax)=1;
      }
    }
  }
	
  /*print*/
  // for(k=index->hstart+1;k<index->hend;k++) 
  //  printf("\n%d\t%d", count[k], *(*finalstarts+k));

  free(tempcount);
  free(count);
  if (CHECKIMG_BYTE(startpoints)) RELIMG_BYTE(&startpoints);
  return;
}

